/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.discovery.model;

import com.google.api.client.json.GenericJson;

/**
 * Model definition for RestMethod.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the APIs Discovery Service. For a detailed explanation
 * see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class RestMethod extends GenericJson {

  /**
   * Description of this method.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String description;

  /**
   * Whether this method requires an ETag to be specified. The ETag is sent as an HTTP If-Match or
   * If-None-Match header.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean etagRequired;

  /**
   * HTTP method used by this method.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String httpMethod;

  /**
   * A unique ID for this method. This property can be used to match methods between different
   * versions of Discovery.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String id;

  /**
   * Media upload parameters.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private MediaUpload mediaUpload;

  /**
   * Ordered list of required parameters, serves as a hint to clients on how to structure their
   * method signatures. The array is ordered such that the "most-significant" parameter appears
   * first.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> parameterOrder;

  /**
   * Details for all parameters in this method.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.Map<String, JsonSchema> parameters;

  static {
    // hack to force ProGuard to consider JsonSchema used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(JsonSchema.class);
  }

  /**
   * The URI path of this REST method. Should be used in conjunction with the basePath property at
   * the api-level.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String path;

  /**
   * The schema for the request.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Request request;

  /**
   * The schema for the response.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Response response;

  /**
   * OAuth 2.0 scopes applicable to this method.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> scopes;

  /**
   * Whether this method supports media downloads.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean supportsMediaDownload;

  /**
   * Whether this method supports media uploads.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean supportsMediaUpload;

  /**
   * Whether this method supports subscriptions.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean supportsSubscription;

  /**
   * Description of this method.
   * The value returned may be {@code null}.
   */
  public String getDescription() {
    return description;
  }

  /**
   * Description of this method.
   * The value set may be {@code null}.
   */
  public RestMethod setDescription(String description) {
    this.description = description;
    return this;
  }

  /**
   * Whether this method requires an ETag to be specified. The ETag is sent as an HTTP If-Match or
   * If-None-Match header.
   * The value returned may be {@code null}.
   */
  public Boolean getEtagRequired() {
    return etagRequired;
  }

  /**
   * Whether this method requires an ETag to be specified. The ETag is sent as an HTTP If-Match or
   * If-None-Match header.
   * The value set may be {@code null}.
   */
  public RestMethod setEtagRequired(Boolean etagRequired) {
    this.etagRequired = etagRequired;
    return this;
  }

  /**
   * HTTP method used by this method.
   * The value returned may be {@code null}.
   */
  public String getHttpMethod() {
    return httpMethod;
  }

  /**
   * HTTP method used by this method.
   * The value set may be {@code null}.
   */
  public RestMethod setHttpMethod(String httpMethod) {
    this.httpMethod = httpMethod;
    return this;
  }

  /**
   * A unique ID for this method. This property can be used to match methods between different
   * versions of Discovery.
   * The value returned may be {@code null}.
   */
  public String getId() {
    return id;
  }

  /**
   * A unique ID for this method. This property can be used to match methods between different
   * versions of Discovery.
   * The value set may be {@code null}.
   */
  public RestMethod setId(String id) {
    this.id = id;
    return this;
  }

  /**
   * Media upload parameters.
   * The value returned may be {@code null}.
   */
  public MediaUpload getMediaUpload() {
    return mediaUpload;
  }

  /**
   * Media upload parameters.
   * The value set may be {@code null}.
   */
  public RestMethod setMediaUpload(MediaUpload mediaUpload) {
    this.mediaUpload = mediaUpload;
    return this;
  }

  /**
   * Ordered list of required parameters, serves as a hint to clients on how to structure their
   * method signatures. The array is ordered such that the "most-significant" parameter appears
   * first.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getParameterOrder() {
    return parameterOrder;
  }

  /**
   * Ordered list of required parameters, serves as a hint to clients on how to structure their
   * method signatures. The array is ordered such that the "most-significant" parameter appears
   * first.
   * The value set may be {@code null}.
   */
  public RestMethod setParameterOrder(java.util.List<String> parameterOrder) {
    this.parameterOrder = parameterOrder;
    return this;
  }

  /**
   * Details for all parameters in this method.
   * The value returned may be {@code null}.
   */
  public java.util.Map<String, JsonSchema> getParameters() {
    return parameters;
  }

  /**
   * Details for all parameters in this method.
   * The value set may be {@code null}.
   */
  public RestMethod setParameters(java.util.Map<String, JsonSchema> parameters) {
    this.parameters = parameters;
    return this;
  }

  /**
   * The URI path of this REST method. Should be used in conjunction with the basePath property at
   * the api-level.
   * The value returned may be {@code null}.
   */
  public String getPath() {
    return path;
  }

  /**
   * The URI path of this REST method. Should be used in conjunction with the basePath property at
   * the api-level.
   * The value set may be {@code null}.
   */
  public RestMethod setPath(String path) {
    this.path = path;
    return this;
  }

  /**
   * The schema for the request.
   * The value returned may be {@code null}.
   */
  public Request getRequest() {
    return request;
  }

  /**
   * The schema for the request.
   * The value set may be {@code null}.
   */
  public RestMethod setRequest(Request request) {
    this.request = request;
    return this;
  }

  /**
   * The schema for the response.
   * The value returned may be {@code null}.
   */
  public Response getResponse() {
    return response;
  }

  /**
   * The schema for the response.
   * The value set may be {@code null}.
   */
  public RestMethod setResponse(Response response) {
    this.response = response;
    return this;
  }

  /**
   * OAuth 2.0 scopes applicable to this method.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getScopes() {
    return scopes;
  }

  /**
   * OAuth 2.0 scopes applicable to this method.
   * The value set may be {@code null}.
   */
  public RestMethod setScopes(java.util.List<String> scopes) {
    this.scopes = scopes;
    return this;
  }

  /**
   * Whether this method supports media downloads.
   * The value returned may be {@code null}.
   */
  public Boolean getSupportsMediaDownload() {
    return supportsMediaDownload;
  }

  /**
   * Whether this method supports media downloads.
   * The value set may be {@code null}.
   */
  public RestMethod setSupportsMediaDownload(Boolean supportsMediaDownload) {
    this.supportsMediaDownload = supportsMediaDownload;
    return this;
  }

  /**
   * Whether this method supports media uploads.
   * The value returned may be {@code null}.
   */
  public Boolean getSupportsMediaUpload() {
    return supportsMediaUpload;
  }

  /**
   * Whether this method supports media uploads.
   * The value set may be {@code null}.
   */
  public RestMethod setSupportsMediaUpload(Boolean supportsMediaUpload) {
    this.supportsMediaUpload = supportsMediaUpload;
    return this;
  }

  /**
   * Whether this method supports subscriptions.
   * The value returned may be {@code null}.
   */
  public Boolean getSupportsSubscription() {
    return supportsSubscription;
  }

  /**
   * Whether this method supports subscriptions.
   * The value set may be {@code null}.
   */
  public RestMethod setSupportsSubscription(Boolean supportsSubscription) {
    this.supportsSubscription = supportsSubscription;
    return this;
  }

  /**
   * Media upload parameters.
   */
  public static final class MediaUpload extends GenericJson {

    /**
     * MIME Media Ranges for acceptable media uploads to this method.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> accept;

    /**
     * Maximum size of a media upload, such as "1MB", "2GB" or "3TB".
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String maxSize;

    /**
     * Supported upload protocols.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Protocols protocols;

    /**
     * MIME Media Ranges for acceptable media uploads to this method.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getAccept() {
      return accept;
    }

    /**
     * MIME Media Ranges for acceptable media uploads to this method.
     * The value set may be {@code null}.
     */
    public MediaUpload setAccept(java.util.List<String> accept) {
      this.accept = accept;
      return this;
    }

    /**
     * Maximum size of a media upload, such as "1MB", "2GB" or "3TB".
     * The value returned may be {@code null}.
     */
    public String getMaxSize() {
      return maxSize;
    }

    /**
     * Maximum size of a media upload, such as "1MB", "2GB" or "3TB".
     * The value set may be {@code null}.
     */
    public MediaUpload setMaxSize(String maxSize) {
      this.maxSize = maxSize;
      return this;
    }

    /**
     * Supported upload protocols.
     * The value returned may be {@code null}.
     */
    public Protocols getProtocols() {
      return protocols;
    }

    /**
     * Supported upload protocols.
     * The value set may be {@code null}.
     */
    public MediaUpload setProtocols(Protocols protocols) {
      this.protocols = protocols;
      return this;
    }

    /**
     * Supported upload protocols.
     */
    public static final class Protocols extends GenericJson {

      /**
       * Supports the Resumable Media Upload protocol.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Resumable resumable;

      /**
       * Supports uploading as a single HTTP request.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Simple simple;

      /**
       * Supports the Resumable Media Upload protocol.
       * The value returned may be {@code null}.
       */
      public Resumable getResumable() {
        return resumable;
      }

      /**
       * Supports the Resumable Media Upload protocol.
       * The value set may be {@code null}.
       */
      public Protocols setResumable(Resumable resumable) {
        this.resumable = resumable;
        return this;
      }

      /**
       * Supports uploading as a single HTTP request.
       * The value returned may be {@code null}.
       */
      public Simple getSimple() {
        return simple;
      }

      /**
       * Supports uploading as a single HTTP request.
       * The value set may be {@code null}.
       */
      public Protocols setSimple(Simple simple) {
        this.simple = simple;
        return this;
      }

      /**
       * Supports the Resumable Media Upload protocol.
       */
      public static final class Resumable extends GenericJson {

        /**
         * True if this endpoint supports uploading multipart media.
         * The value may be {@code null}.
         */
        @com.google.api.client.util.Key
        private Boolean multipart;

        /**
         * The URI path to be used for upload. Should be used in conjunction with the basePath property at
         * the api-level.
         * The value may be {@code null}.
         */
        @com.google.api.client.util.Key
        private String path;

        /**
         * True if this endpoint supports uploading multipart media.
         * The value returned may be {@code null}.
         */
        public Boolean getMultipart() {
          return multipart;
        }

        /**
         * True if this endpoint supports uploading multipart media.
         * The value set may be {@code null}.
         */
        public Resumable setMultipart(Boolean multipart) {
          this.multipart = multipart;
          return this;
        }

        /**
         * Convenience method that returns only {@link Boolean#TRUE} or {@link Boolean#FALSE}.
         *
         * <p>
         * Boolean properties can have four possible values:
         * {@code null}, {@link com.google.api.client.util.Data#NULL_BOOLEAN}, {@link Boolean#TRUE}
         * or {@link Boolean#FALSE}.
         * </p>
         *
         * <p>
         * This method returns {@link Boolean#TRUE} if the default of the property is {@link Boolean#TRUE}
         * and it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
         * {@link Boolean#FALSE} is returned if the default of the property is {@link Boolean#FALSE} and
         * it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
         * </p>
         *
         * <p>
         * True if this endpoint supports uploading multipart media.
         * </p>
         */
        public boolean isMultipart() {
          if (multipart == null || multipart == com.google.api.client.util.Data.NULL_BOOLEAN) {
            return true;
          }
          return multipart;
        }

        /**
         * The URI path to be used for upload. Should be used in conjunction with the basePath property at
         * the api-level.
         * The value returned may be {@code null}.
         */
        public String getPath() {
          return path;
        }

        /**
         * The URI path to be used for upload. Should be used in conjunction with the basePath property at
         * the api-level.
         * The value set may be {@code null}.
         */
        public Resumable setPath(String path) {
          this.path = path;
          return this;
        }

      }
      /**
       * Supports uploading as a single HTTP request.
       */
      public static final class Simple extends GenericJson {

        /**
         * True if this endpoint supports upload multipart media.
         * The value may be {@code null}.
         */
        @com.google.api.client.util.Key
        private Boolean multipart;

        /**
         * The URI path to be used for upload. Should be used in conjunction with the basePath property at
         * the api-level.
         * The value may be {@code null}.
         */
        @com.google.api.client.util.Key
        private String path;

        /**
         * True if this endpoint supports upload multipart media.
         * The value returned may be {@code null}.
         */
        public Boolean getMultipart() {
          return multipart;
        }

        /**
         * True if this endpoint supports upload multipart media.
         * The value set may be {@code null}.
         */
        public Simple setMultipart(Boolean multipart) {
          this.multipart = multipart;
          return this;
        }

        /**
         * Convenience method that returns only {@link Boolean#TRUE} or {@link Boolean#FALSE}.
         *
         * <p>
         * Boolean properties can have four possible values:
         * {@code null}, {@link com.google.api.client.util.Data#NULL_BOOLEAN}, {@link Boolean#TRUE}
         * or {@link Boolean#FALSE}.
         * </p>
         *
         * <p>
         * This method returns {@link Boolean#TRUE} if the default of the property is {@link Boolean#TRUE}
         * and it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
         * {@link Boolean#FALSE} is returned if the default of the property is {@link Boolean#FALSE} and
         * it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
         * </p>
         *
         * <p>
         * True if this endpoint supports upload multipart media.
         * </p>
         */
        public boolean isMultipart() {
          if (multipart == null || multipart == com.google.api.client.util.Data.NULL_BOOLEAN) {
            return true;
          }
          return multipart;
        }

        /**
         * The URI path to be used for upload. Should be used in conjunction with the basePath property at
         * the api-level.
         * The value returned may be {@code null}.
         */
        public String getPath() {
          return path;
        }

        /**
         * The URI path to be used for upload. Should be used in conjunction with the basePath property at
         * the api-level.
         * The value set may be {@code null}.
         */
        public Simple setPath(String path) {
          this.path = path;
          return this;
        }

      }
    }
  }

  /**
   * The schema for the request.
   */
  public static final class Request extends GenericJson {

    /**
     * Schema ID for the request schema.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String $ref;

    /**
     * Schema ID for the request schema.
     * The value returned may be {@code null}.
     */
    public String get$ref() {
      return $ref;
    }

    /**
     * Schema ID for the request schema.
     * The value set may be {@code null}.
     */
    public Request set$ref(String $ref) {
      this.$ref = $ref;
      return this;
    }

  }

  /**
   * The schema for the response.
   */
  public static final class Response extends GenericJson {

    /**
     * Schema ID for the response schema.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String $ref;

    /**
     * Schema ID for the response schema.
     * The value returned may be {@code null}.
     */
    public String get$ref() {
      return $ref;
    }

    /**
     * Schema ID for the response schema.
     * The value set may be {@code null}.
     */
    public Response set$ref(String $ref) {
      this.$ref = $ref;
      return this;
    }

  }

}
